@@ -1,13 +1,16 @@
-"""凭据类，用于请求验证"""
+"""凭据类,用于请求验证"""
 
-from dataclasses import dataclass, field
-from typing import Union
+import json
+import sys
+from dataclasses import asdict, dataclass, field
+from typing import Any
 
-from qqmusic_api.exceptions import (
-    CredentialNoMusicidException,
-    CredentialNoMusickeyException,
-    CredentialNoRefreshkeyException,
-)
+if sys.version_info >= (3, 11):
+    from typing import Self
+else:
+    from typing_extensions import Self
+
+from ..exceptions import CredentialInvalidError
 
 
 @dataclass
@@ -39,7 +42,7 @@ class Credential:
     refresh_key: str = ""
     encrypt_uin: str = ""
     login_type: int = 0
-    extra_fields: dict[str, Union[str, int]] = field(default_factory=dict)
+    extra_fields: dict[str, Any] = field(default_factory=dict)
 
     def __post_init__(self):
         if not self.login_type:
@@ -49,75 +52,67 @@ class Credential:
                 self.login_type = 2
 
     def has_musicid(self) -> bool:
-        """是否提供 musicid
-
-        Returns:
-            是否提供
-        """
+        """是否提供 musicid"""
         return bool(self.musicid)
 
     def has_musickey(self) -> bool:
-        """是否提供 musickey
-
-        Returns:
-            是否提供
-        """
+        """是否提供 musickey"""
         return bool(self.musickey)
 
-    def can_refresh(self) -> bool:
-        """是否能刷新 Credential
-
-        Returns:
-            是否能刷新
-        """
-        return bool(self.musicid) and (bool(self.refresh_key) or bool(self.musickey))
-
-    def raise_for_cannot_refresh(self):
-        """无法刷新 Credential 时抛出异常
+    def raise_for_invalid(self) -> None:
+        """检查凭据是否有效
 
         Raises:
-            CredentialNoRefreshkeyException
-        """
-        if not self.can_refresh():
-            raise CredentialNoRefreshkeyException()
-
-    def raise_for_no_musicid(self):
-        """没有提供 musicid 时抛出异常
-
-        Raises:
-            CredentialNoMusicidException
+            CredentialInvalidError: 没有提供 musicid 或 musickey
         """
         if not self.has_musicid():
-            raise CredentialNoMusicidException()
+            raise CredentialInvalidError("没有提供 musicid")
 
-    def raise_for_no_musickey(self):
-        """没有提供 musickey 时抛出异常"""
         if not self.has_musickey():
-            raise CredentialNoMusickeyException()
+            raise CredentialInvalidError("没有提供 musickey")
 
-    async def refresh(self):
+    async def refresh(self) -> bool:
         """刷新 cookies"""
         from ..login import refresh_cookies
 
-        c = await refresh_cookies(self)
-        self.__dict__.update(c.__dict__)
+        return await refresh_cookies(self)
+
+    async def can_refresh(self) -> bool:
+        """是否可以刷新 credential"""
+        if not self.has_musicid() or not self.has_musickey():
+            return False
+        if await self.is_expired():
+            return bool(self.refresh_key) and bool(self.refresh_token)
+        return True
+
+    async def is_expired(self) -> bool:
+        """判断 credential 是否过期"""
+        from ..login import check_expired
+
+        return await check_expired(self)
+
+    def as_dict(self) -> dict:
+        """获取凭据字典"""
+        d = asdict(self)
+        d["loginType"] = d.pop("login_type")
+        d["encryptUin"] = d.pop("encrypt_uin")
+        return d
+
+    def as_json(self) -> str:
+        """获取凭据 JSON 字符串"""
+        data = self.as_dict()
+        data.update(data.pop("extra_fields"))
+        return json.dumps(data, indent=4, ensure_ascii=False)
 
     @classmethod
-    def from_cookies(cls, cookies: dict) -> "Credential":
-        """从 cookies 创建 Credential 实例
-
-        Args:
-            cookies: Cookies 字典
-
-        Returns:
-            凭据类实例
-        """
+    def from_cookies_dict(cls, cookies: dict[str, Any]) -> Self:
+        """从 cookies 字典创建 Credential 实例"""
         return cls(
             openid=cookies.pop("openid", ""),
             refresh_token=cookies.pop("refresh_token", ""),
             access_token=cookies.pop("access_token", ""),
             expired_at=cookies.pop("expired_at", 0),
-            musicid=cookies.pop("musicid", ""),
+            musicid=cookies.pop("musicid", 0),
             musickey=cookies.pop("musickey", ""),
             unionid=cookies.pop("unionid", ""),
             str_musicid=cookies.pop(
@@ -129,3 +124,8 @@ class Credential:
             login_type=cookies.pop("loginType", 0),
             extra_fields=cookies,
         )
+
+    @classmethod
+    def from_cookies_str(cls, cookies: str) -> Self:
+        """从 cookies 字符串创建 Credential 实例"""
+        return cls.from_cookies_dict(json.loads(cookies))
